import { __decorate, __param } from 'tslib';
import { Input, HostListener, Directive, Inject, ElementRef, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';

let NgxPatternDirective = class NgxPatternDirective {
    constructor(host, document) {
        this.host = host;
        this.document = document;
    }
    ngOnChanges(changes) {
        if (this.ngxPattern) {
            if (typeof this.ngxPattern === 'string') {
                this.regExPattern = new RegExp(`^${this.ngxPattern}$`, 'g');
            }
            else {
                this.regExPattern = this.ngxPattern;
            }
        }
    }
    onKeyDown(event) {
        if (this.regExPattern && noKeyFlags(event) && !isSpecialKey(event.key)) {
            if (!this.validWithChange(event.key)) {
                event.preventDefault();
            }
        }
    }
    onInput() {
        if (!this.textIsValid(this.currentValue)) {
            // Mobile browsers don't support keydown preventDefault and return
            // Unidentified for the pressed key. We need to detect the change on input event and undo.
            this.document.execCommand('undo');
        }
    }
    onPaste(event) {
        if (event.clipboardData) {
            const pastedInput = event.clipboardData.getData('text/plain');
            if (!this.validWithChange(pastedInput)) {
                event.preventDefault();
            }
        }
    }
    onDrop(event) {
        if (event.dataTransfer) {
            const textData = event.dataTransfer.getData('text');
            if (!this.validWithChange(textData)) {
                event.preventDefault();
            }
        }
    }
    get currentValue() {
        return this.inputEl ? this.inputEl.value : undefined;
    }
    validWithChange(delta) {
        const current = this.inputEl.value;
        const selectionStart = this.inputEl.selectionStart || 0;
        const selectionEnd = this.inputEl.selectionEnd || 0;
        const updated = current.substring(0, selectionStart) + delta + current.substring(selectionEnd + 1);
        return this.textIsValid(updated);
    }
    textIsValid(text) {
        if (this.regExPattern) {
            const result = !text || this.regExPattern.test(text);
            this.regExPattern.lastIndex = 0;
            return result;
        }
        return true;
    }
    get inputEl() {
        return this.host.nativeElement;
    }
};
__decorate([
    Input()
], NgxPatternDirective.prototype, "ngxPattern", void 0);
__decorate([
    HostListener('keydown', ['$event'])
], NgxPatternDirective.prototype, "onKeyDown", null);
__decorate([
    HostListener('input', [])
], NgxPatternDirective.prototype, "onInput", null);
__decorate([
    HostListener('paste', ['$event'])
], NgxPatternDirective.prototype, "onPaste", null);
__decorate([
    HostListener('drop', ['$event'])
], NgxPatternDirective.prototype, "onDrop", null);
NgxPatternDirective = __decorate([
    Directive({
        selector: '[ngxPattern]'
    }),
    __param(0, Inject(ElementRef)),
    __param(1, Inject(DOCUMENT))
], NgxPatternDirective);
function noKeyFlags(event) {
    return !event.ctrlKey && !event.altKey && !event.metaKey;
}
/** @see https://developer.mozilla.org/bg/docs/Web/API/KeyboardEvent/key/Key_Values */
function isSpecialKey(key) {
    return key.length > 1;
}

let NgxPatternModule = class NgxPatternModule {
};
NgxPatternModule = __decorate([
    NgModule({
        declarations: [NgxPatternDirective],
        imports: [],
        exports: [
            NgxPatternDirective
        ]
    })
], NgxPatternModule);

/*
 * Public API Surface of ngx-pattern
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxPatternDirective, NgxPatternModule };
//# sourceMappingURL=ngx-pattern.js.map
